function GetSPData()
	local p = GetCN("spdata")
	if p and p.data then 
		return p.data 
	else
		return nil
	end
end

function GetSPVar(var)
	local p = GetSPData()
	if not (var and type(var)=="string") then 
		return nil
	end
	if p and (type(p)=="table") and p[var] then 
		return p[var]
	else
		return nil
	end
end

function ShowOptionsMenu()
	TheFrontEnd:PushScreen(SPOptionsScreen())
end

function KeyShowOptionsMenu()
	if not IsPaused() and not TheInput:IsKeyDown(KEY_CTRL) and TheInput:IsKeyDown(KEY_ALT) and not TheInput:IsKeyDown(KEY_SHIFT) then
		ShowOptionsMenu()
	end
end

function ChangeKeyDown(key)
	if not (key and (type(key)=="number")) then
		print ("ChangeKeyDown : invalid key ("..tostring(key)..")")
		return 
	end
	if KeyDownHandler and KeyDownCode and (key == KeyDownCode) then 
		print ("ChangeKeyDown : same key ("..tostring(key).."), skipping...")
		return 
	end
	if KeyDownHandler then TheInput.onkeydown:RemoveHandler(KeyDownHandler) end
	if (key == 0 ) or (key < KEY_A) or (key > KEY_Z) then 
		KeyDownCode = nil
		KeyDownHandler = nil
	else
		KeyDownCode = key
		KeyDownHandler = TheInput:AddKeyDownHandler(KeyDownCode, KeyShowOptionsMenu)
	end
end

function ApplyCustomData()
	local p = GetSPData()
	local player = GetPlayer()
	if p then

		local awidget = 1   -- active pos
		local cwidget = 10  -- custom pos
		local dwidget = 20  -- disabled pos
		for k,v in pairs(spWidgetOrder) do  -- resort widgets ID, disabled to end
--			print("Sort - spWidget["..tostring(k).."] = "..tostring(v))
			d = p[spWidgetName[k]]
			if d then
				if d.data then
--					print ("Cur = "..tostring(awidget))
					if d.custompos then 
						spWidgetOrder[k].wid = cwidget
						cwidget = cwidget + 1
					else
						spWidgetOrder[k].wid = awidget
						awidget = awidget + 1
					end
				else
--					print ("Dis = "..tostring(dwidget))
					spWidgetOrder[k].wid = dwidget
					dwidget = dwidget + 1
				end	
			end
		end

		for k,v in pairs(spWidget) do
			if p[k] then
				local d = deepcopy(spdata["default"])
				d.ws = p.scale
				d.bsx = 0.9 * p.barscale
				d.fsx = d.bsx + 0.08
				
				if p[k].wx then 
					d.wx = p[k].wx 
				elseif spWidget[k].wd  then
					d.wx = spWidget[k].wd.wx 
				end

				spWidget[k]:SetWData(d)
				
				if p[k].custompos then 
					if p[k].wy then 
						d.wy = p[k].wy 
					elseif spWidget[k].wd  then
						d.wy = spWidget[k].wd.wy 
					end
				else
					d.wx = spWidget[k]:GetInitX()
					d.wy = spWidget[k]:GetInitY()
				end

				spWidget[k]:SetWData(d)
				spWidget[k]:SetWColor(player.components.spdata:GetColor())
				spWidget[k]:ApplyData()
				spWidget[k]:SetFGTint("g")
				spWidget[k]:ReDraw()
				spWidget[k]:ShowWidget(p[k].data)
			else
				print ("ApplyCustomData for "..tostring(k).." failed")
			end
		end
		if player and player.HUD and player.HUD.controls and player.HUD.controls.status then
			local st = player.HUD.controls.status
			local stw = nil
			for k,v in ipairs({"playerstats","monsterstats","playermagic"})do
				if p[v] then
					stw = st[v]
					if stw and p[v].data then
						stw:Show()
					else
						if stw then stw:Hide() end
					end
				end
			end
		end
		if p.keycode then
			ChangeKeyDown(p.keycode)
		end
	end
end

function ResetCustomWidgets()
	local p = GetSPData()
	if p then
		for k,v in pairs(spWidget) do
			if p[k] then
				p[k].custompos = false
			end
		end
	end
	ApplyCustomData()
end

function RedrawWidgets()
	for k,v in pairs(spWidget) do
		spWidget[k]:ReDraw()
	end
end
